/*
Branch and bound integer program solver. Easy enough to use, probably a tad slow?

A reasonable TODO:
	Pull the B&B tree into a class of its own (so we can do RINS and so on)
	Put the branching into a class of its own (again for more flexibility).
Most interesting for folk at school would be an "interactive mode".

(c) Matthew Steel 2009

This program is free software. It comes without any warranty, to
the extent permitted by applicable law. You can redistribute it
and/or modify it under the terms of the Do What The Fuck You Want
To Public License, Version 2, as published by Sam Hocevar. See
http://sam.zoy.org/wtfpl/COPYING for more details.
*/

#ifndef BRANCH_AND_BOUND_HPP
#define BRANCH_AND_BOUND_HPP

#include <vector>
#include <stack>
#include "Simplex.hpp"

class BranchAndBound
{
	public:
		BranchAndBound(Simplex, const std::vector<bool>&);
		void solve(double tolerance=1e-8);
		double currentObjective() const { return upperBound; }
		bool provedInfeasible() const { return done && !feasible; }
		//Should just return objective == -infinity?
		bool isOptimal() const { return done && feasible; }

	private:
		void removeCurrentNode();
		bool isFeasible(const Simplex&, double, unsigned&);

		Simplex optimalSolution;
		std::stack<Simplex> activeNodes;
		std::stack<double> lowerBoundStack;
		/*
		We're doing a DFS through the B&B tree. Lower bound is the best-valued, non-expanded simplex in the tree. Using two stacks so we can push/pop nodes while keeping track of the lower bound.
		*/
		
		std::vector<bool> forceInteger;
		//variable constraints
		
		double upperBound;//Best IP solution found
		bool feasible;
		bool done;
};


#endif
